/* Iria - An easy-customizing game, in the way of "Thunder Force IV".
 * Created by Juan Jos Ramrez. Synthetyx Software.
 *
 * graphedit.c - This provides an interface for creating and editing images 
 * 	that could be used in the game.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "graphedit.h"

char currentcolor = BLACK; // Current color for drawing
char please_draw = 1; // True if the image has changed
char please_exit = 0; // True if we want to exit 
char zoom = 1; // Zoom over work area
int workOffsetX = 0; // Offset in work area (when the image isn't in (0,0) position)
int workOffsetY = 0;
int selectX1 = 0, selectY1 = 0, selectX2 = 0, selectY2 = 0; // Variables for image selection
GraphicsContext physicalscreen; // I follow the variable names given in the vgagl manual
png_uint_32 width, height; // Width and height of the image loaded
char new_file = 0;
png_color pal[256];	// For setting RGB values


#define GEN1(var, fix1, fix2, init, end, val1, val2, mul)	\
	for (i = init; i < end; i++) {			\
		pal[i + 1].var = (i - init) * mul;			\
		pal[i + 1].fix1 = val1;			\
		pal[i + 1].fix2 = val2;			\
	}
#define GEN2(var, fix1, fix2, init, end, val, mul1, mul2) 	\
	for (i = init; i < end; i++) {				\
		pal[i + 1].var = val;				\
		pal[i + 1].fix1 = (i - init) * mul1;			\
		 pal[i + 1].fix2 = (i - init) * mul2;			\
	}							

/* We will do some graphics inicializations in order to be able to draw things in the screen */
void graph_initializations (void **expandedfont) {
	int auxpal[256 * 3], i;
	
	vga_init();
	vga_setmode(GRAPHMODE);
	vga_setpalette(0, 0, 30, 30);	// This will be the transparent color

	gl_setcontextvga(GRAPHMODE);
	gl_getcontext(&physicalscreen); 
	gl_setcontextvgavirtual(GRAPHMODE);
	gl_clearscreen(BLACK);
	gl_setclippingwindow(0, 0, WIDTH, HEIGHT);
	gl_enableclipping();
	
	*expandedfont = (void *) malloc(256 * 8 * 8 * BYTESPERPIXEL);		
	gl_expandfont(8, 8, TITLECOLOR, gl_font8x8, *expandedfont);
	gl_setfont(8, 8, *expandedfont);
	gl_setwritemode(WRITEMODE_MASKED); // The text background is transparent

	// Palette initialization
	pal[0].blue = 0; // Transparent color
	pal[0].green = 30;
	pal[0].red = 0;

	GEN1(blue, green, red, 0x0, 0x8, 0, 0, 8);
	GEN2(blue, green, red, 0x8, 0x10, 63, 8, 8);
	GEN1(green, blue, red, 0x10, 0x18, 0, 0, 8);
	GEN2(green, blue, red, 0x18, 0x20, 63, 8, 8);
	GEN1(red, green, blue, 0x20, 0x28, 0, 0, 8);
	GEN2(red, green, blue, 0x28, 0x30, 63, 8, 8);
	GEN2(green, blue, red, 0x30, 0x38, 0, 8, 8);			
	GEN1(green, blue, red, 0x38, 0x40, 63, 63, 8);	
	GEN2(blue, blue, green, 0x40, 0x48, 63, 8, 8);		
	GEN1(red, blue, green, 0x48, 0x50, 63, 63, 8);
	GEN2(red, red, green, 0x50, 0x58, 63, 8, 8);
	GEN1(blue, green, red, 0x58, 0x60, 63, 63, 8);	
	GEN2(green, green, red, 0x60, 0x68, 0, 4, 8);
	GEN1(blue, green, red, 0x68, 0x70, 31, 63, 8);
	GEN1(blue, red, green, 0x70, 0x78, 31, 63, 8);
	GEN1(green, blue, red, 0x78, 0x80, 31, 63, 8);
	GEN1(green, red, blue, 0x80, 0x88, 31, 63, 8);

	for (i = 0x89; i < 0x91; i++)
		pal[i].blue = pal[i].green = pal[i].red = (i - 0x89) * 8;
	pal[0x91].blue = pal[0x91].green = pal[0x91].red = 63;

	
	for (i = 0; i < 256; i++) {
		auxpal[i * 3] = pal[i].red;
		auxpal[(i * 3) + 1] = pal[i].green;
		auxpal[(i * 3) + 2] = pal[i].blue;
	}
	vga_setpalvec(0, 256, auxpal);	
}

/* And now, some mouse routines... */
void mouse_initializations () {
	vga_setmousesupport(1);
	// I must change this to make it compatible with all mouses
	mouse_init("/dev/mouse", MOUSE_MICROSOFT, MOUSE_DEFAULTSAMPLERATE);
	mouse_setxrange(0, SCREEN_WIDTH);
	mouse_setyrange(0, SCREEN_HEIGHT);
}

/* Finally, let's do something with the keyboard... */
void keyboard_initializations() {
	if (keyboard_init()) {
		printf("\nI have some problems with the keyboard... aborting\n");
		exit(1);
	}
}

/* Basically, it puts a lot of zeros 8) */
void init_image(column **image) {
	int i;
	column *ptr;	
	if ( !(*image = (column *) malloc(width * sizeof(column))) ) {
		printf("\nI haven't enough memory... aborting\n");
		exit(NOT_ENOUGH_MEMORY);
	}
	ptr = *image;
	for (i = 0; i < width; i++) {
		if ( !(ptr[i] = (column) calloc(height, sizeof(char))) ) {
			printf("\nI haven't enough memory... aborting\n");
			exit(NOT_ENOUGH_MEMORY);
		}
	}
}

/* This function is the opposite to init_image */
void free_image(column *image, int count) {
	int i;
	for (i = 0; i < count; i++)
		free(image[i]);
	free(image);
}


/*
 * This function changes rows for columns because PNG format stores the file in 'row order' instead
 * 'column order' used in graphedit
 */
int swap_rows(png_bytep **s, column **d, int width, int height) {
	png_bytep *ptr;
	int x,  y;
	
	if (!(*d = (png_bytep *) malloc(width * sizeof(png_bytep)))) return NOT_ENOUGH_MEMORY;
	ptr = *d;
	for (x = 0; x < width; x++) 
		if (!(ptr[x] = (png_bytep) malloc(height * sizeof(png_byte)))) return NOT_ENOUGH_MEMORY;
	for (x = 0; x < width; x++)
		for (y = 0; y < height; y++)
			(*d)[x][y] = (*s)[y][x];
	free_image(*s, height);
	return GOODBYE_OK;
}
	

/* This function reads the image or create a new one */
void read_the_image(char *filename, column **image) {
	png_bytep *aux;
	switch (readpng(filename, &aux, &width, &height)) {
		case FILE_DOESNT_EXIST: 
			new_file = 1;
			printf("File doesn't exist. Creating a new file...\nWidth?: ");
			scanf("%lu", &width);
			printf("Height?: ");
			scanf("%lu", &height);
			init_image(image);
			break;
		case NOT_ENOUGH_MEMORY:
			printf("\nI haven't enough memory... aborting\n");
			exit(NOT_ENOUGH_MEMORY);
			break;
		case GOODBYE_OK:
			if (swap_rows(&aux, image, width, height) == NOT_ENOUGH_MEMORY) {
				printf("\nI haven't enough memory... aborting\n");
				exit(NOT_ENOUGH_MEMORY);
			}
			break;
	}

}
	
/* This function ask the user if he/she wants to save changes. If so, we will write it */
void write_the_image(char *filename, column *image) {
	png_bytep *swapped;
	int save, x;	
	
	if (!new_file) {
		printf("\nSave changes? [Y/N]");
		save = getchar();
		printf("\n");		
	}
	else
		save = 'Y';
	if ((save == 'Y') || (save == 'y')) {
		for (x = 0; x < 256; x++) {	// PNG format uses a (0-255) RGB values (instead of (0-63) of svgalib) and blue and red colours are swapped
			pal[x].blue *= 4;
			pal[x].green *= 4;
			pal[x].red *= 4;
		}
		swap_rows(&image, &swapped, height, width);
		writepng(filename, swapped, width, height, pal);
		free_image(swapped, height);
	}
	else
		free_image(image, width);
}

/* Draw the palette colors */
void drawpalette() {
	int i;
	// Drawing palette colors
	for (i = 0; i < MAXPALCOLORS; i++) { 
		gl_fillbox(palX + (i / palrows * palboxsize), palY + (i % palrows * palboxsize), palboxsize, palboxsize, BLACK);
		gl_fillbox(palX + (i / palrows * palboxsize), palY + (i % palrows * palboxsize), palcolorsize, palcolorsize, i);
	}
}
/* This function draws the work area where you draw the pictures */
void DrawWorkArea() {
	gl_fillbox(workX, workY, workwidth, workheight, WHITE);
	gl_fillbox(workX, workY, workwidth - 1, workheight - 1, BLACK);
}

/* This function draws the pointer you use to draw or select things */
void drawpointer(int x, int y) {
	static int oldx = 10, oldy = 10;
	// I'll create an extern module for allocate (and exit if not) memory
	if ((oldx != x) || (oldy != y)) {
		oldx = x; oldy = y;
		// ...and we redraw the new cursor at new position
		vga_setcolor(WHITE);
		vga_drawline(x - 5, y, x - 1, y); // Left-horizontal
		vga_drawline(x + 1, y, x + 5, y); // Right-horizontal
		vga_drawline(x, y - 5, x, y - 1); // Up-vertical
		vga_drawline(x, y + 1, x, y + 5); // Down-vertical
		vga_setcolor(currentcolor);
		vga_drawpixel(x, y);
		/* I will do 2 more black lines for each white line, rounding them */
	}
}


/* Draws the current color box which indicates the color you will draw */
void drawcurrentcolorbox() {
	gl_fillbox(currcolX, currcolY, currcolsize, currcolsize, WHITE);
	gl_fillbox(currcolX + 1, currcolY + 1, currcolsize - 2, currcolsize - 2, currentcolor);
}

/* This function draws all the buttons */
void drawbuttons() {
	/* Zoom button */
	gl_fillbox(zoombuttonX, zoombuttonY, buttonwidth, buttonheight, buttoncolor);
	gl_write(zoombuttonX + 1, zoombuttonY + 1, "Zoom");
	/* Center button */
	gl_fillbox(centerbuttonX, centerbuttonY, buttonwidth, buttonheight, buttoncolor);
	gl_write(centerbuttonX + 1, centerbuttonY + 1, "Center");
	/* Center button */
	gl_fillbox(clearbuttonX, clearbuttonY, buttonwidth, buttonheight, buttoncolor);
	gl_write(clearbuttonX + 1, clearbuttonY + 1, "Clear");
	/* Copy button */
	gl_fillbox(copybuttonX, copybuttonY, buttonwidth, buttonheight, buttoncolor);
	gl_write(copybuttonX + 1, copybuttonY + 1, "Copy");
	/* Exit button */
	gl_fillbox(exitbuttonX, exitbuttonY, buttonwidth, buttonheight, buttoncolor);
	gl_write(exitbuttonX + 1, exitbuttonY + 1, "Exit");
}	

/* This function draws the selection box (if there are any selected) */
void draw_selected() {
	int X1, X2, Y1, Y2;
	if ((selectX1 != selectX2) && (selectY1 != selectY2)) {
		X1 = (selectX1 - workOffsetX) * zoom;
		X2 = (selectX2 - workOffsetX) * zoom;
		Y1 = (selectY1 - workOffsetY) * zoom;
		Y2 = (selectY2 - workOffsetY) * zoom;
		if (X1 < 0)
			X1 = 0;
		if (X1 > MAXWIDTH)
			X1 = MAXWIDTH;
		if (Y1 < 0)
			Y1 = 0;
		if (Y1 > MAXHEIGHT)
			Y1 = MAXHEIGHT;
		if (X2 < 0)
			X2 = 0;
		if (X2 > MAXWIDTH)
			X2 = MAXWIDTH;
		if (Y2 < 0)
			Y2 = 0;
		if (Y2 > MAXHEIGHT)
			Y2 = MAXHEIGHT;
		vga_drawline(X1, Y1, X2, Y1);
		vga_drawline(X2, Y1, X2, Y2);
		vga_drawline(X1, Y1, X1, Y2);		
		vga_drawline(X1, Y2, X2, Y2);
	}
}

/*
 * It tests if the mouse is over any color of the palette. If it is, it get
 * the new color
 */
void getcurrentcolor(int x, int y) {
	char aux;
	if (mouse_getbutton()) {
		aux = ((y - palY) / palboxsize) + (((x - palX) / palboxsize) * palrows);
		if (aux != currentcolor) {
			currentcolor = aux;
			drawcurrentcolorbox();
		}
	}
}

/* Return where is the pointer (palette, work area...)*/
char getpossition(int x, int y) {
	if ((x >= palX) && (x <= palX + palwidth) && (y >= palY) && (y < palY + palheight)) 
		return ON_PALETTE;
	else if
	  ((x >= workX) && (x  < workX + workwidth ) && (y >= workY) && (y <= workY + workheight)) 
		return ON_WORK;
	else if 
	  ((x >= zoombuttonX) && (x < zoombuttonX + buttonwidth) && (y >= zoombuttonY) && (y < zoombuttonY + buttonheight))
		return ON_ZOOM;
	else if 
	  ((x >= centerbuttonX) && (x < centerbuttonX + buttonwidth) && (y >= centerbuttonY) && (y < centerbuttonY + buttonheight))
		return ON_CENTER;
	else if 
	  ((x >= clearbuttonX) && (x < clearbuttonX + buttonwidth) && (y >= clearbuttonY) && (y < clearbuttonY + buttonheight))
		return ON_CLEAR;
	else if 
	  ((x >= copybuttonX) && (x < copybuttonX + buttonwidth) && (y >= copybuttonY) && (y < copybuttonY + buttonheight))
		return ON_COPY;
	else if 
	  ((x >= exitbuttonX) && (x < exitbuttonX + buttonwidth) && (y >= exitbuttonY) && (y < exitbuttonY + buttonheight))
		return ON_EXIT;
	else	return ON_NONE;
}


/* It redraws the image in the work area */
void redraw_image(column *image) {
#define EFFECTIVE_I (i / zoom) + workOffsetX
#define EFFECTIVE_J (j / zoom) + workOffsetY
	int i, j;
	for (i = 0; (i < MAXWIDTH) && (i < workwidth); i++) 
		for (j = 0; (j < MAXHEIGHT) && (j < workheight); j++) 			
			if ((EFFECTIVE_I < width) && (EFFECTIVE_J < height) && (EFFECTIVE_I >= 0) && (EFFECTIVE_J >= 0))  // If we are drawing the image, not the outsides
				gl_setpixel(i, j, image[EFFECTIVE_I][EFFECTIVE_J]);
			else gl_setpixel(i, j, (i + j) % 2? WHITE : BLACK);
}	

/* For optimization, we only redraw the pixel previously drew. This is a job for 'update_image' */
void update_image(column *image, int xpos, int ypos) {
	int x, y;
	x = ((xpos - workX) / zoom);
	y = ((ypos - workY) / zoom);
	gl_fillbox(x * zoom, y * zoom, (x * zoom + zoom < MAXWIDTH)? zoom:MAXWIDTH - x * zoom, (y * zoom + zoom < MAXHEIGHT)? zoom:MAXHEIGHT - y * zoom, image[x + workOffsetX][y + workOffsetY]);	
	please_draw = 0;
}


/* When user is over work area, 'hitwork' manages it */
void hitwork(column *image, int x, int y, int mousebutton) {
	static char first_time = 1;	// If TRUE, is the first time the user press middle button, else the user has move the mouse (and, then, the image)
	static int firstX, firstY;

	x /= zoom;  // We adjust the zoom factor to find out the exact position of the image matrix
	y /= zoom;
	switch (mousebutton) {
		case  MOUSE_LEFTBUTTON:	// User wants to draw a pixel
			if ((x + workOffsetX < width) && (y + workOffsetY < height) && (x + workOffsetX >= 0) && (y + workOffsetY >= 0))
				if ((please_draw = (image[x + workOffsetX][y + workOffsetY] != currentcolor))) 
					image[x + workOffsetX][y + workOffsetY] = currentcolor;				
			break;
		case MOUSE_MIDDLEBUTTON:	// User wants to get the pixelcolor
			if ((x + workOffsetX < width) && (y + workOffsetY < height) && (x + workOffsetX >= 0) && (y + workOffsetY >= 0))
				if (image[x][y] != currentcolor) {
					currentcolor = image[x][y];
					drawcurrentcolorbox();
				}
			break;
		case MOUSE_RIGHTBUTTON:  // User wants to move the image
			if (first_time) {
				firstX = x;
				firstY = y;
				first_time = 0;
			}
			else {
				workOffsetX -= x - firstX;
				workOffsetY -= y - firstY;
				firstX = x;
				firstY = y;
				redraw_image(image);
			}
			break;
		default: first_time = 1;			
	}
}

/* If press LEFT mousebutton, increases the zoom, if RIGTH decreases it */
int hitzoom(int mousebutton) {
	switch (mousebutton) {
		case MOUSE_LEFTBUTTON: 	if (zoom < 16) {
						zoom++;
						return REDRAW;
					}
					break;
		case MOUSE_RIGHTBUTTON: if (zoom > 1) {
						zoom--;
						return REDRAW;
					}
					break;
	}
	return NOTHING;
}

/* If the center button is hit, the image will be at (0,0) */
void hitcenter(int mousebutton, column *image)  {
	if (mousebutton == MOUSE_LEFTBUTTON) {
		workOffsetX = 0;
		workOffsetY = 0;
		redraw_image(image);
	}
}

/*
 * If we hit 'clear' button, the selected image will turn into transparent 
 * color 
 */
void hitclear(int mousebutton, column *image) {
	int x, y;
	if (mousebutton == MOUSE_LEFTBUTTON) {
		for (x = selectX1; x < selectX2; x++)
			for (y = selectY1; y < selectY2; y++)
				image[x][y] = 0;
		redraw_image(image);
	}
}

/* If we hits over 'exit' button, please_exit will be true */
void hitexit(int mousebutton) {
	if (mousebutton == MOUSE_LEFTBUTTON) 
		please_exit = 1;
}

/* This function makes a selection from the image for copying, saving... */
void select_image() {
	selectX1 = selectX2 = mouse_getx() - (mouse_getx() % zoom);
	selectY1 = selectY2 = mouse_gety() - (mouse_gety() % zoom);
	vga_setcolor(WHITE);
	while (mouse_getbutton()) {
		mouse_waitforupdate();
		selectX2 = mouse_getx() - (mouse_getx() % zoom);
		selectY2 = mouse_gety() - (mouse_gety() % zoom);
		gl_copyscreen(&physicalscreen);
		vga_drawline(selectX1, selectY1, selectX2, selectY1);
		vga_drawline(selectX2, selectY1, selectX2, selectY2);
		vga_drawline(selectX1, selectY1, selectX1, selectY2);		
		vga_drawline(selectX1, selectY2, selectX2, selectY2);
	}
	if ((selectX1 == selectX2) || (selectY1 == selectY2)) {	// None has been selected, so ALL is selected
		selectX1 = selectY1 = 0;
		selectX2 = width;
		selectY2 = height;
	}
	else {	// A nice selection
		selectX1 = (selectX1 * zoom) + workOffsetX;
		selectX2 = (selectX2 * zoom) + workOffsetX;
		selectY1 = (selectY1 * zoom) + workOffsetY;
		selectY2 = (selectY2 * zoom) + workOffsetY;
	}
}

/* This function changes the palette when the SIGALRM is sent */
static void change_palette(int sig) {
	static char col252, col253, col254, col255; // Colors to be changed in palette-rotation	

	 vga_setpalette(255, col255, col255, col255);	// A Gray-fade
	 col255 = (col255 % 64) + 1;
	 
	 vga_setpalette(254, col254, 0, 0);	// A Red-Fade
	 col254 = (col254 % 64) + 1;	 

	 vga_setpalette(253, 0, col253, 0);	// A Green-Fade
	 col253 = (col253 % 64) + 1;	 	

	 vga_setpalette(252, 0, 0, col252);	// A Blue-Fade
	 col252 = (col252 % 64) + 1;	 
	 
} 
	

void configure_the_clock() {
	struct itimerval mytime;

	signal(SIGALRM, change_palette); // When the alarm signal is sent, we change the palette
	mytime.it_interval.tv_sec = 0;
	mytime.it_interval.tv_usec = PAL_DELAY;
	mytime.it_value.tv_sec = 0;
	mytime.it_value.tv_usec = PAL_DELAY;
	setitimer(ITIMER_REAL, &mytime, NULL);
}
	
	
		
int main(int argc, char **argv) {
	column *image;
	void * expandedfont; // For font manipulation...
	
	if (argc < 2) {
		printf("You must specify a png file to edit...\n");
		exit(1);
	}

	// Please, see, see, SEE and LEARN jmp stuff... Try reading row by row
	read_the_image(argv[1], &image);
	graph_initializations(&expandedfont);
	mouse_initializations();
	keyboard_initializations();
	configure_the_clock();	

	
	drawpalette();
	DrawWorkArea();
	drawcurrentcolorbox();
	drawbuttons();
	gl_write(TITLEX,TITLEY, VERSION);	
	redraw_image(image);
	gl_copyscreen(&physicalscreen);	 // The first draw
	drawpointer(mouse_getx(), mouse_gety());	

			
	while (!please_exit) {				
		gl_copyscreen(&physicalscreen);		
		drawpointer(mouse_getx(), mouse_gety());
		switch ( getpossition(mouse_getx(), mouse_gety()) ) {
			case ON_PALETTE: getcurrentcolor(mouse_getx(), mouse_gety());
					break;
			case ON_WORK:	if (keyboard_keypressed(SCANCODE_LEFTSHIFT)) 
								select_image();
							else 
								hitwork(image, mouse_getx(), mouse_gety(), mouse_getbutton());
					break;
			case ON_ZOOM:	if (hitzoom(mouse_getbutton()))
						redraw_image(image);
					break;
			case ON_CENTER: hitcenter(mouse_getbutton(), image);
					break;
			case ON_CLEAR: 	hitclear(mouse_getbutton(), image);
					break;
			case ON_EXIT:	hitexit(mouse_getbutton());
					break;
		}
		if (please_draw) update_image(image, mouse_getx(), mouse_gety());
		draw_selected();
		mouse_waitforupdate(); // Waiting for an event...	 
		keyboard_update();
	}	
	mouse_close();
	keyboard_close();
	vga_setmode(TEXT);	
	write_the_image(argv[1], image);
	free(expandedfont);

	return GOODBYE_OK;
}

