/* Iria - An easy-customizing game, in the way of "Thunder Force IV".
 * Created by Juan Jos Ramrez. Synthetyx Software.
 *
 * lists.h - Prototypes and definitions for lists.c
 *
 */

#include <malloc.h>
#include "returns.h"
#include "structs.h"

typedef struct string_node {
	char info[STRING_LENGTH];
	struct string_node *next;
} string_node;
typedef string_node *string_list;


/* Every list needs to be initialized. This function inits a new list */
extern void string_new(string_list *l);

/* This function adds an element to the list */
extern int string_insert(string_list *l, char *s);

/* This function extracts an element from the list */
extern void string_extract(string_list *l, char *s); 

/* This function unloads all elements from memory */
extern void string_destroy(string_list *l);

/*
 * This functions returns 1 if the string is equal to one or more elements of
 * the list. Else, it returns 0
 */
extern int string_find(string_list l, char *s);

/* This function inits a new image file list */
extern void imagefile_new(imagefile_list *l);

/* This function inserts a new image file node */
extern int imagefile_insert(imagefile_list *l, char *filename, image_list image);

/* This function unloads all elements from memory */
extern void imagefile_destroy(imagefile_list *l);

/*
 * This functions returns 1 if the filename is equal to one or more filenames of
 * the list. Else, it returns 0
 */
extern int imagefile_find(imagefile_list l, char *filename);

/* This function inits a new image point list */
extern void imagepoint_new(imagepoint_list *l);

/* This function inserts a new image point node */
extern int imagepoint_insert(imagepoint_list *l, unsigned long int point, char *filename, png_bytep image, png_uint_32 width, png_uint_32 height);

/* This function unloads all elements from memory */
extern void imagepoint_destroy(imagepoint_list *l);

/*
 * This functions returns 1 if the filename is equal to one or more filenames of
 * the list. Else, it returns 0
 */
extern int imagepoint_find(imagepoint_list l, char *filename);

/* This function inits a new list */
extern void eventpoint_new(eventpoint_list *l);

/* This function loads a new element to the list, ordered by point */
extern int eventpoint_insert(eventpoint_list *l, event_record node);

/* This function replaces an element */
extern void eventpoint_replace(eventpoint_list l, event_record old, event_record new);

/* This function deletes an element */
extern void eventpoint_delete(eventpoint_list *l, event_record event);

/* This function deletes all elements from the list */
extern void eventpoint_destroy(eventpoint_list *l);

/* This function initializes a new event list */
extern void event_new(event_list *l);

/* This function inserts a new element to the list */
extern int event_insert(event_list *l, event_record data, void *pointer, unsigned long int object_id);

/* This function unloads all elements from memory */
extern void event_destroy(event_list *l);

/* This function inits a new image list */
extern void image_new(image_list *l); 

/* This function inserts a new element to the image list */
extern int image_insert(image_list *l, png_bytep image, png_uint_32 width, png_uint_32 height);

/* This function unloads all elements from memory */
extern void image_destroy(image_list *l);

/* This function will create a new list */
extern void counter_new(counter_list *l);

/* This will insert a new element */
extern int counter_insert(counter_list *l, unsigned long int point, unsigned long int count);

/*
 * This functions returns 1 if point is found in the list.
 * Else, it returns 0
 */
extern int counter_find(counter_list l, unsigned long int point);

/* This function will returns the value of the counter in a point */
extern unsigned long int counter_value(counter_list l, unsigned long int point);

/* This function will decrease in 1 unit the value of the counter */
extern void counter_decrease(counter_list l, unsigned long int point);

/* This function looks for an element and deletes it */
extern void counter_delete(counter_list *l, unsigned long int point);

/* This function deletes all elements from the list */
extern void counter_destroy(counter_list *l);

/* This function inits a new list */
extern void weapon_new(id_list *l);

/* This function inserts a new element to the list */
extern int weapon_insert(id_list *l, unsigned long int id, int x, int y);

/* This function deletes the first weapon */
extern void weapon_delete(id_list *l);

/* This function deletes all elements from the list */
extern void weapon_destroy(id_list *l);

/* This function inits a new list */
extern void object_new(object_list *l);

/* This function inserts a new element to the list, ordered by layer (from object_index) */
extern int object_insert(object_list *l, object_node data, object_struct *object_index);

/* This function finds and deletes an element from the list */
extern void object_delete(object_list *l, object_list element);

/* This function deletes all elements from the list */
extern void object_destroy(object_list *l);

/* This function inits a new list */
extern void dead_new(dead_list *l);

/* This function inserts a new element to the list */
extern int dead_insert(dead_list *l, object_list new);

/* This function deletes all elements from the list */
extern void dead_destroy(dead_list *l);
