/* Iria - An easy-customizing game, in the way of "Thunder Force IV".
 * Created by Juan Jos Ramrez. Synthetyx Software.
 *
 * mngevents.c - Manage events. Very useful module for showing and 
 *	filling events.
 *
 */

#include "mngevents.h"

/* This function fills the inmortality and life section */
void change_inmortality(int *inmortal, unsigned long int *life) {
	char option;
	printf("Is inmortal? [Y/N] ");
	scanf("%c", &option);
	*inmortal = (option == 'y') || (option == 'Y');
	if (!*inmortal) {				
		printf("Type life: ");
		scanf("%ld", life);
	}
}

/* This function fills movement section */
void change_movement(int *move_type, char *move_file, int *loop) {
	int option;
	do {
		printf("Select movement style:\n"
			"1) Filename specification\n"
			"2) To ship\n"
			"3) To left\n"
			"4) To right\n"
			"5) None\n>");
		scanf("%d", move_type);
	} while ((*move_type < 1) || (*move_type > 5));
	if (*move_type == 1) {
		printf("Type filename for movement (*.mov): ");
		scanf("%s", move_file);
		printf("Movement is a loop?: [Y/N] ");
		do option = getchar(); while (option == 10);
		*loop = (option == 'y') || (option == 'Y');
	}
	else {
		printf("Object speed: ");
		scanf("%d", loop);
	}
}

/* This function fills shoot section */
void change_shoot(int *shoot_type, unsigned long int *count) {
	do {
		printf("Select shoot style:\n"
			"1) Always\n"
			"2) Random\n"
			"3) Vertical at same vertical position\n"
			"4) Horizontal at same horizontal position\n"
			"5) Every n times\n"
			"6) None\n>");
		scanf("%d", shoot_type);
	} while ((*shoot_type < 1) || (*shoot_type > 6));
	if (*shoot_type == 5) {
		printf("Type times: ");
		scanf("%lu", count);
	}
}


/* This function fill the fields of an event_record */
void fill_data(event_record *data, char use_point) {
	char c;

	if (use_point) {
		printf("What point?: ");
		scanf("%lu", &data->point);
	}
	do {
		printf("\nSelect an event type:\n"
			"1) Unconditional jump to other point\n"
			"2) Jump if count is 0\n"
			"3) Jump keeping in mind enemy count\n"
			"4) Show a message\n"
			"5) Change a scenery parameter\n"						
			"6) Create an object\n"
#ifndef SCENERY_HEADER
			"7) Change object image\n"
			"8) Change life and inmortality\n"
			"9) Change move behaviour\n"
			"10) Change shoot behaviour\n"
			"11) Object erase\n"
			"12) Add weapon\n"
#endif
			">");
		scanf("%d", &data->type);
#ifndef SCENERY_HEADER				
	} while ((data->type < UNCONDITIONAL_JUMP) || (data->type > LAST_EVENT_TYPE));
#else
	} while ((data->type < UNCONDITIONAL_JUMP) || (data->type > CREATE_OBJECT));			
#endif
	switch (data->type) {	
		case JUMP_WITH_COUNT: 
		case JUMP_ENEMY_COUNT:	printf("Type count: ");
								scanf("%lu", &data->count);
		case UNCONDITIONAL_JUMP:printf("Where to jump?: ");
								scanf("%lu", &data->aux);
			break;
		case SHOW_MESSAGE:	printf("Write a nice message (no more than %d characters)\n>", STRING_LENGTH);
							fgets(data->string, STRING_LENGTH, stdin);
			break;
		case CHANGE_PARAMETER:	printf("What do you want to change?:\n"
										"1) Background Image\n"
										"2) Background type\n"
										"3) Palette change\n>");
								do scanf("%lu", &data->aux); while ((data->aux < 1) || (data->aux > 3));
								switch(data->aux) {
									case 1: printf("Type image file: ");
											scanf("%s", data->string);
										break;
									case 2: select_bg((int *) &data->count);
										break;
									case 3: printf("Select color to change (0-255): ");
											scanf("%d", (int *) &data->string[0]);
											printf("Select Red (0-63): ");
											scanf("%d", (int *) &data->string[1]);
											printf("Select Green (0-63): ");
											scanf("%d", (int *) &data->string[2]);
											printf("Select Blue (0-63): ");
											scanf("%d", (int *) &data->string[3]);
										break;
								}
			break;
		case CREATE_OBJECT: printf("Filename?: ");
							scanf("%s", data->string);
							if (use_point) {
								printf("What height?: ");
								scanf("%lu", &data->aux);
							}
							printf("Controled by player (0 for machine): ");
							scanf("%lu", &data->count);
			break;
		case CHANGE_OBJECT_IMAGE:	printf("Type image file for object (non-existing file for none): ");
									scanf("%s", data->string);
			break;
		case CHANGE_LIFE: change_inmortality((int *)&data->aux, &data->count);
			break;
		case CHANGE_MOVEMENT: change_movement((int *) &data->aux, data->string, (int *) &data->count);
			break;
		case CHANGE_SHOOT: change_shoot((int *) &data->aux, &data->count);
			break;
		case OBJECT_ERASE:	printf("Do you want an explosion? [Y/N] ");
							scanf("%c", &c);
							data->aux = ((c = 'y') || (c = 'Y'));
			break;
		case ADD_WEAPON:	printf("Type weapon object file: ");
							scanf("%s", data->string);
							printf("Type weapon offset in x axis: ");
							scanf ("%d", (int *) &data->aux);
							printf("Type weapon offset in y axis: ");
							scanf ("%d", (int *) &data->count);
			break;
	}
}

/* This function return the correct position (Remember: file is ordered by point) */
unsigned long int find_file_position(FILE *f, unsigned long int p) {
	event_record aux;
	unsigned long int pos;
	
	SKIP_HEADER;
	do {
		fgetpos(f, &pos);		
		fread(&aux, sizeof(event_record), 1, f);
	} while ((aux.point < p) && (!feof(f)));
	return pos;
}


/* This function selects a background type */
void select_bg(int *bg) {
	printf("Choose a background type for the scenery\n"
	"1) None\n"
	"2) Stars\n"
	"3) Snow\n>");
	do {
		scanf("%d", bg);
	} while (( *bg < 1) || (*bg > 3));
}


/* This function outputs the data info in listing */
void print_data(event_record data) {
	static const char *event[] = {
		"Unconditional jump ",
		"Jump with count ",
		"Jump with enemy count ",
		"Show ",
		"Scenery parameter change: ",
		"Insert object ",
		"Object image ",
		"Life: ",
		"Movement: ",
		"Shoot: ",
		"Object erase ",
		"Add weapon "
	};
	static const char *change[] = {
		"Background Image ",
		"Background type ",
		"Palette change "
	};
	static const char *bg_type[] = {		
		"none",
		"stars",
		"snow"
	};		
	static const char *move_type[] = {				
		"Filename specification",
		"To ship",
		"To left",
		"To right",
		"None"
	};
	static const char *shoot_type[] = {
		"Always",
		"Random",
		"Vertical at same vertical position",
		"Horizontal at same horizontal position",
		"Every ",
		"None"
	};
		
	
	printf("%s", event[data.type - 1]);
	switch (data.type) {
		case JUMP_WITH_COUNT:
		case JUMP_ENEMY_COUNT: printf("%lu", data.count);
		case UNCONDITIONAL_JUMP: printf(" to %lu\n", data.aux);
			break;
		case CREATE_OBJECT: printf("at height %d controled by player %lu, file name: ", (int) data.aux, data.count);
		case SHOW_MESSAGE: 
		case CHANGE_OBJECT_IMAGE:	printf("\"%s\"\n", data.string);
			break;
		case CHANGE_PARAMETER:	printf("%s", change[data.aux - 1]);
								switch (data.aux) {
									case 1: printf("%s\n", data.string);
										break;
									case 2: printf("%s\n", bg_type[data.count - 1]);
										break;
									case 3: printf("color %d with RGB: %d %d %d\n", data.string[0], data.string[1], data.string[2], data.string[3]);
										break;
								}
			break;
		case CHANGE_LIFE:	if (!data.aux) // if it's inmortal...
								printf("Inmortality set\n");
							else
								printf("%lu units\n", data.count); // Life
			break;
		case CHANGE_MOVEMENT:	if (data.aux == 1) { // Filename specification
								printf("\"%s\"", data.string);
								if (data.count) // Loop enabled
									printf(" with loop\n");
								else
									printf(" without loop\n");
							}
							else
								printf("%s with speed %lu\n", move_type[data.aux], data.count);
			break;
		case CHANGE_SHOOT:	if (data.aux != 4)	// != n times
								printf("%s, object \"%s\"\n", shoot_type[data.aux], data.string);
							else 
								printf("%s %d times, object \"%s\"\n", shoot_type[data.aux], (int) data.count, data.string);
			break;
		case OBJECT_ERASE:	if (data.aux)
								printf("with explosion\n");
							else
								printf("without explosion\n");
			break;
		case ADD_WEAPON:	printf("\"%s\"\n", data.string);
	}
}

/* This function shows scenery global values */
void show_scenery_globals(FILE *f) {
	scn_header header;	
	static const char *bg_type[] = {		
		"none",
		"stars",
		"snow"
	};		
	

	fseek(f, 0, SEEK_SET);
	fread(&header, sizeof(scn_header), 1, f);
	printf("Scenery version %s\n"
		"Scenery height: %d\n"
		"Scenery speed: %d\n"
		"Scenery initial point: %lu\n"
		"Background effect: %s\n"
		"Background image (non-existing file for none): %s\n", header.version, header.height, header.speed, header.init_point, bg_type[header.bg_type - 1], header.filename);
}

/* This function shows object global values */
void show_object_globals(FILE *f) {
	obj_header header;
	static const char *move_type[] = {
		"filename specification",
		"to ship",
		"to left",
		"to right",
		"none"
	};	
	static const char *shoot_type[] = {
		"always",
		"random",
		"vertical at same vertical position",
		"horizontal at same horizontal position",
		"every",
		"none"
	};
		

	fseek(f, 0, SEEK_SET);
	fread(&header, sizeof(obj_header), 1, f);
	printf("Object version %s\n"
		"Main image file \"%s\"\n"
		"\"Go up\" image file \"%s\"\n"
		"\"Go down\" image file \"%s\"\n"
		"Shoot image file \"%s\"\n", header.version, header.image, header.upimage_file, header.downimage_file, header.shootimage_file);
	printf("Damage power: %lu\n", header.damage);
	if (header.inmortal) 
		printf("It's inmortal\n");
	else
		printf("Life: %ld\n", header.life);
	if (header.move_type == 1) 
		printf("Move file \"%s\"\n", header.move_file);
	else 
		printf("Move behaviour %s\n", move_type[header.move_type - 1]);
	if (header.move_type == 1) {
		if (header.move_aux)
			printf("Looping enabled\n");
		else
			printf("Looping disabled\n");
	}
	else
		printf("Object speed %d\n", header.move_aux);
	printf("Shoot behaviour %s", shoot_type[header.shoot_type - 1]);
	if (header.shoot_type == 4)
		printf(" %lu times\n", header.shoot_count);
	else
		printf("\n");
	printf("Energy needed for creating object: %lu\n", header.shoot_energy);
	if (header.damage) 
		printf("Hit object \"%s\"\n"
				"Die object \"%s\"\n"
				"Goodie object \"%s\"\n",  header.hit_file, header.die_file, header.goodies);
	printf("Object layer height %d\n", header.layer);
}

/* This function shows all events numbered in a point. It is useful for selecting one event */
unsigned long int show_events_in_a_point(FILE *f, unsigned long int p) {
	event_record data;
	unsigned long int count = 0;
	
	fseek(f, find_file_position(f, p), SEEK_SET);
	fread(&data, sizeof(event_record), 1, f);
	if (data.point == p) {
		do {
			count++;
			printf("\t%lu) ", count);
			print_data(data);
			fread(&data, sizeof(event_record), 1, f);
		} while ((data.point == p) && !feof(f));
		return count;	
	}
	else return 0;	
}
	
/* This function list events in a point */
void list_events(FILE *f, char *return_menu) {
	event_record data;
	int option;
	unsigned long int last = 0xfffffff, count = 0;
	do {
		do {
			printf("\nLISTING MENU\nSelect an option:\n"
				"1) Show all events in all points\n"
				"2) Show all points\n"
				"3) Show all events in a point\n"
				"4) Show globals\n"
				"5) Exit from listing and return to %s menu\n>", return_menu);
			scanf("%d", &option);
		} while ((option < 1) || (option > 5));
		SKIP_HEADER;
		switch (option) {
			case 1: while (fread(&data, sizeof(event_record), 1, f)) {
					if (data.point != last) {
						count = 0;						
						printf("The point %lu has following events\n", data.point);
						last = data.point;
					}
					count++;
					printf("\t%lu) ", count);
					print_data(data);					
				}
				if (!count) printf("There aren't points\n");
			case 2: while (fread(&data, sizeof(event_record), 1, f)) {
					if (data.point != last) {
						printf("\tPoint %lu\n", data.point);
						last = data.point;
						count++;
					}
				}
				if (!count) printf("There aren't points\n");
				break;
			case 3: printf("Type a point: "); 
				scanf("%lu", &last);	// Here, 'last' is the point we are looking for
				printf("\nPoint %lu\n", last);					
				if (!show_events_in_a_point(f, last)) printf("That point doesn't exist\n");
				break;
			case 4:
#ifdef SCENERY_HEADER
				show_scenery_globals(f);
#else
				show_object_globals(f);
#endif
				break;
		}
	} while (option != 5);
}

