/* Iria - An easy-customizing game, in the way of "Thunder Force IV".
 * Created by Juan Jos Ramrez. Synthetyx Software.
 *
 * object_editor.c - This provides a single interface for creating and
 *	editing new objects, with its events.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <string.h>
#include "textevents.h"	// For editing, deleting, inserting, listing... events with text interface
#include "returns.h"

#define OBJECT_VERSION "v0.3.5"



/* This function loads the general data for an object */
void load_general(FILE *f) {
	int option;
	header_record header;
	
	do {
		printf("Type image file for object (non-existing file for none): ");
		scanf("%s", header.image);
		printf("Type object image file when go up (non-existing file for none): ");
		scanf("%s", header.upimage_file);
		printf("Type object image file when go down (non-existing file for none): ");
		scanf("%s", header.downimage_file);
		printf("Type object image file when shooting (non-existing file for none): ");
		scanf("%s", header.shootimage_file);
		change_inmortality(&header.inmortal, &header.life);
		printf("Type object damage power: ");
		scanf("%lu", &header.damage);
		printf("Type object layer height: ");
		scanf("%d", &header.layer);
		printf("Type object energy needed for creating it: ");
		scanf("%lu", &header.shoot_energy);
		change_movement(&header.move_type, header.move_file, &header.move_aux);
		change_shoot(&header.shoot_type, &header.shoot_count);
		if (header.damage) {
			printf("Type hit object file (non-existing for none): ");
			scanf("%s", header.hit_file);
			printf("Type die object file (non-existing for none): ");
			scanf("%s", header.die_file);
		}
		printf("Type goodie object (non-existing for none): ");
		scanf("%s", header.goodies);
		printf("Is all correct? [Y/N] ");
		do option = getchar(); while (option == 10);
	} while ((option != 'Y') && (option != 'y'));

	strcpy(header.version, OBJECT_VERSION);
	rewind(f);
	fwrite(&header, sizeof(header_record), 1, f);
}


/* This function will load or create an object file */
void load_a_file(FILE **f, char *filename) {
	header_record header;
	if (!(*f = fopen(filename, "rb+"))) {
		*f = fopen(filename, "wb+");
		printf("This is a new file, so I will ask you some questions.\nSee README.objects for help.\n");		
		load_general(*f);
	}
	else {
		fread(&header, sizeof(header_record), 1, *f);
		if (strcmp(header.version, OBJECT_VERSION)) {
			printf("This file has a different version (%s).\nNeeded version %s. Aborting...\n", header.version, OBJECT_VERSION);
			fclose(*f);
			exit(FILE_DOESNT_EXIST);
		}
	}
}


int main(int argc, char **argv) {
	FILE *f;
	int option;
	if (argc < 2) {
		printf("You must specify a file to edit (*.iri)\n");
		return FILE_DOESNT_EXIST;
	}
	load_a_file(&f, argv[1]);
	while ((option = select_menu(f)) != SELECT_EXIT) {
		switch (option) {
			case 1: load_general(f);
				break;
			case 2: edit_event(f);
				break;
			case 3: insert_event(f);
				break;
			case 4: delete_event(&f, argv[1]);
				break;
			case 5: list_events(f, "MAIN");
				break;
		}
	}
	fclose(f);
	return GOODBYE_OK;
}
