/* Iria - An easy-customizing game, in the way of "Thunder Force IV".
 * Created by Juan Jos Ramrez. Synthetyx Software.
 *
 * scenery_editor.c - This provides a single interface for creating and
 *	editing new sceneries, with its events.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include "textevents.h"	// For editing, deleting, inserting, listing... events with text interface
#include "returns.h"

#define SCENERY_VERSION "v0.6.0"

		

/* This function loads the general data for an object */
void load_general(FILE *f) {
	int option;
	header_record header;
	
	do {
		printf("What height do you want for the scenery?: ");
		scanf("%d", &header.height);
		printf("What speed do you want for the scenery?: ");
		scanf("%d", &header.speed);
		printf("What is the initial point for the scenery?: ");
		scanf("%lu", &header.init_point);
		do {
			printf("Choose a background effect for the scenery: \n"
				"1) None\n"
				"2) Stars\n"
				"3) Snow\n>");
			scanf("%d", &header.bg_type);
		} while ((header.bg_type < 1) || (header.bg_type > 3));
		printf("Type an image file for background (non-existing file for none): ");
		scanf("%s", header.filename);
		printf("Is all correct? [Y/N] ");
		do option = getchar(); while (option == 10);
	} while ((option != 'Y') && (option != 'y'));

	strcpy(header.version, SCENERY_VERSION);
	rewind(f);
	fwrite(&header, sizeof(header_record), 1, f);
}


/* This function will load or create a scenery file */
void load_a_file(FILE **f, char *filename) {
	header_record header;
	if (!(*f = fopen(filename, "rb+"))) {
		*f = fopen(filename, "wb+");
		printf("This is a new file, so I will ask you some questions.\nSee README.scenery for help.\n");		
		load_general(*f);
	}
	else {
		fread(&header, sizeof(header_record), 1, *f);
		if (strcmp(header.version, SCENERY_VERSION)) {
			printf("This file has a different version (%s).\nNeeded version %s. Aborting...\n", header.version, SCENERY_VERSION);
			fclose(*f);
			exit(FILE_DOESNT_EXIST);
		}
	}
}


int main(int argc, char **argv) {
	FILE *f;
	int option;
	if (argc < 2) {
		printf("You must specify a file to edit (*.scn)\n");
		return FILE_DOESNT_EXIST;
	}
	load_a_file(&f, argv[1]);
	while ((option = select_menu(f)) != SELECT_EXIT) {
		switch (option) {
			case 1: load_general(f);
				break;
			case 2: edit_event(f);
				break;
			case 3: insert_event(f);
				break;
			case 4: delete_event(&f, argv[1]);
				break;
			case 5: list_events(f, "MAIN");
				break;
		}
	}
	fclose(f);
	return GOODBYE_OK;
}
