/* Iria - An easy-customizing game, in the way of "Thunder Force IV".
 * Created by Juan Jos Ramrez. Synthetyx Software.
 *
 * svgalibg.c - A module for managing svgalib routines
 *
 */

#include "svgalibg.h"

/* This function gets the selected video mode */
int get_graphics_mode() {
	return DEFAULT_GRAPHICS_MODE;
}

/* This function makes important graphics initializations */
int svga_graphics_initializations(GraphicsContext *phy, GraphicsContext *virt, void **expandedfont, int height) {
	char error;
	int graphmode, i, *auxpal;
	png_color pal[256];	

	graphmode = get_graphics_mode();
	error = vga_init(); 
	if (error) return GRAPHS_ERROR;
	
	error = vga_setmode(graphmode);
	if (error) return GRAPHS_ERROR;

	error = gl_setcontextvga(graphmode);
	if (error) return GRAPHS_ERROR;
	
	gl_getcontext(phy);
	error = gl_setcontextvgavirtual(graphmode);
	if (error) return GRAPHS_ERROR;
	gl_getcontext(virt);
	gl_clearscreen(0);
	gl_enableclipping();
	gl_setclippingwindow(0, 0, WIDTH - 1, HEIGHT < height? HEIGHT - 1 : height - 1);

	*expandedfont = (void *) malloc(256 * 8 * 8 * BYTESPERPIXEL);
	if (!*expandedfont) 
		return NOT_ENOUGH_MEMORY;
	gl_expandfont(8, 8, 15, gl_font8x8, *expandedfont);
	gl_setfont(8, 8, *expandedfont);
	gl_setwritemode(WRITEMODE_MASKED); // The text background is transparent

	// Palette initialization
	pal[0].blue = 0; // Transparent color
	pal[0].green = 0;
	pal[0].red = 0;

	GEN1(blue, green, red, 0x0, 0x8, 0, 0, 8);
	GEN2(blue, green, red, 0x8, 0x10, 63, 8, 8);
	GEN1(green, blue, red, 0x10, 0x18, 0, 0, 8);
	GEN2(green, blue, red, 0x18, 0x20, 63, 8, 8);
	GEN1(red, green, blue, 0x20, 0x28, 0, 0, 8);
	GEN2(red, green, blue, 0x28, 0x30, 63, 8, 8);
	GEN2(green, blue, red, 0x30, 0x38, 0, 8, 8);			
	GEN1(green, blue, red, 0x38, 0x40, 63, 63, 8);	
	GEN2(blue, blue, green, 0x40, 0x48, 63, 8, 8);		
	GEN1(red, blue, green, 0x48, 0x50, 63, 63, 8);
	GEN2(red, red, green, 0x50, 0x58, 63, 8, 8);
	GEN1(blue, green, red, 0x58, 0x60, 63, 63, 8);	
	GEN2(green, green, red, 0x60, 0x68, 0, 4, 8);
	GEN1(blue, green, red, 0x68, 0x70, 31, 63, 8);
	GEN1(blue, red, green, 0x70, 0x78, 31, 63, 8);
	GEN1(green, blue, red, 0x78, 0x80, 31, 63, 8);
	GEN1(green, red, blue, 0x80, 0x88, 31, 63, 8);

	for (i = 0x89; i < 0x91; i++)
		pal[i].blue = pal[i].green = pal[i].red = (i - 0x89) * 8;
	pal[0x91].blue = pal[0x91].green = pal[0x91].red = 63;

	if (!(auxpal = (int *) malloc(256 * 3 * sizeof(int)))) {
		free(*expandedfont);
		return NOT_ENOUGH_MEMORY;
	}
	
	for (i = 0; i < 256; i++) {
		auxpal[i * 3] = pal[i].red;
		auxpal[(i * 3) + 1] = pal[i].green;
		auxpal[(i * 3) + 2] = pal[i].blue;
	}
	
	vga_setpalvec(0, 256, auxpal);	

	free(auxpal);	
/*
	// Palette initialization
	if (!(pal = (int *) malloc(256 * 3 * sizeof(int)))) {
		free(*expandedfont);
		return NOT_ENOUGH_MEMORY;
	}
	for (i = 0; i < 0xE0; i++) {
		pal[(i + 1) * 3] = i < 0x06? (i ^ 0x07) * 10 : (i ^ 0x03) * 10;
		pal[(i + 1) * 3 + 1] = i >= 0x06? ((i ^ 0x1c) >> 2) * 10 : ((i ^ 0x19) >> 2) * 10;
		pal[(i + 1) * 3 + 2] = i < 0xC0? ((i ^0xE0) >> 5) * 10 : ((i ^0x60) >> 5) * 10;
	}
	pal[0] = 0; // Transparent color
	pal[1] = 0;
	pal[2] = 0;
	for (i = 0xE0; i < 0xF0; i++) 
		pal[i * 3] = pal[(i * 3) + 1] = pal[(i * 3) + 2] = (i - 0xE0) * 4;
	vga_setpalvec(0, 256, pal);	
*/	
	return GOODBYE_OK;
}

/* This function returns to text mode and free some variables */
void svga_graphics_shutdown(GraphicsContext *virt, void **expandedfont) {
	gl_freecontext(virt);
	free(*expandedfont);
	vga_setmode(TEXT);
}

/* This function makes keyboard initializations */
int svga_keyboard_initializations() {
	if (keyboard_init())
		return KEYBOARD_ERROR;
	else
		return GOODBYE_OK;
}

/* This function returns keyboard to normal mode */
void svga_keyboard_shutdown() {
	keyboard_close();
}

/* This function makes a mouse initialization */
int svga_mouse_initializations () {
	int result;

	vga_setmousesupport(1);
	// I must change this to make it compatible with all mouses
	result = mouse_init("/dev/mouse", MOUSE_MICROSOFT, MOUSE_DEFAULTSAMPLERATE);
	if (result)
		return MOUSE_ERROR;
	mouse_setxrange(0, WIDTH);
	mouse_setyrange(0, HEIGHT);
	return GOODBYE_OK;
}

/* This function shuts down the mouse */
void svga_mouse_shutdown() {
	mouse_close();
}

/* This function makes svga initializations like video mode or keyboard */
void svga_initializations(GraphicsContext *phy, GraphicsContext *virt, void **expandedfont, int height) {

	if (svga_graphics_initializations(phy, virt, expandedfont, height) != GOODBYE_OK) {
		vga_setmode(TEXT);
		printf("There were errors with graphics initialization!\n");
		exit(GRAPHS_ERROR);
	}
	if (svga_keyboard_initializations() != GOODBYE_OK) {
		svga_graphics_shutdown(virt, expandedfont);

		printf("There were errors with keyboard initialization!\n");
		exit(KEYBOARD_ERROR);
	}
	if (svga_mouse_initializations() != GOODBYE_OK) {
		svga_keyboard_shutdown();
		svga_graphics_shutdown(virt, expandedfont);

		printf("There were errors with mouse initialization!\n");
		exit(MOUSE_ERROR);
	}
}

/* This function shuts down graphics and keyboard raw mode */
void svga_shutdown(GraphicsContext *virt, void **expandedfont) {
	svga_mouse_shutdown();
	svga_keyboard_shutdown();
	svga_graphics_shutdown(virt, expandedfont);
}
