/* Iria - An easy-customizing game, in the way of "Thunder Force IV".
 * Created by Juan Jos Ramrez. Synthetyx Software.
 *
 * textevents.c - Very useful module for creating, inserting, deleting, editing
 *	and listing events, but with a text interface
 *
 */

#include "textevents.h"

/* This function shows a menu and returns the choice */
int select_menu() {
	int option;

	do {
		printf("\nMAIN MENU\nSelect an option:\n"
			"1) Change global settings\n"
			"2) Edit an existing event\n"
			"3) Insert a new event\n"
			"4) Delete an event\n"
			"5) List events\n"
			"6) Exit\n>");
		scanf("%d", &option);
	} while ((option < 1) || (option > SELECT_EXIT));
	return option;
}

/* This function shows an useful menu, common to many others. Returns FALSE if the user wants to exit */
int pre_menu(FILE *f, char *menutype) {
	int option;
	do {
		printf("\n%s MENU\nSelect an option:\n"
			"1) Select point\n"
			"2) List events\n"
			"3) Exit from %s\n>", menutype, menutype);
		scanf("%d", &option);
		if (option == 2) list_events(f, menutype);
	} while (option == 2);
	return option == 1;
}	

/* This function edits an event */
void edit_event(FILE *f) {
	event_record data;
	int option, events;
	
	while (pre_menu(f, "EDITING")) {
		fill_data(&data, POINT);
		if ((events = show_events_in_a_point(f, data.point))) {
			do {
				printf("Select one event to change: ");
				scanf("%d", &option);
			} while ((option < 1) || (option > events));
			fseek(f, find_file_position(f, data.point) + (option - 1) * sizeof(event_record), SEEK_SET);
			fwrite(&data, sizeof(event_record), 1, f);
		}
		else printf("That point doesn't exists!\n");
	}
}
		
	
/* This function inserts a new event */
void insert_event(FILE *f) {
	event_record data, aux;
	unsigned long int insert_position, pos;

	while (pre_menu(f, "INSERTING")) {
		fill_data(&data, POINT);
		insert_position = find_file_position(f, data.point);
		fseek(f, 0, SEEK_END);	
		for (fgetpos(f, &pos); pos != insert_position; pos-=sizeof(event_record)) {
			fseek(f, pos - sizeof(event_record), SEEK_SET);
			fread(&aux, sizeof(event_record), 1, f);
			fseek(f, pos, SEEK_SET);
			fwrite(&aux, sizeof(event_record), 1, f);
		}
		fseek(f, insert_position, SEEK_SET);
		fwrite(&data, sizeof(event_record), 1, f);
	}
}
	
/* This function deletes an event */
void delete_event(FILE **f, char *filename) {
	event_record data;
	int option, events;
	long int pos;
	char end = 0;
	
	while (pre_menu(*f, "DELETING")) {
		printf("What point? ");
		scanf("%lu", &data.point);
		if ((events = show_events_in_a_point(*f, data.point))) {
			do {
				printf("Select one event to delete: ");
				scanf("%d", &option);
			} while ((option < 1) || (option > events));
			fseek(*f, find_file_position(*f, data.point) + (option - 1) * sizeof(event_record), SEEK_SET);
			while (!end) {
				fseek(*f, sizeof(event_record), SEEK_CUR);
				end = fread(&data, sizeof(event_record), 1, *f) == 0;
				if (!end) {
					fseek(*f, -2 * sizeof(event_record), SEEK_CUR);
					fwrite(&data, sizeof(event_record), 1, *f);
				}
			}
			fseek(*f, -sizeof(event_record), SEEK_END);
			pos = ftell(*f);
			fclose(*f);
			truncate(filename, pos);
			*f = fopen(filename, "rb+");
		}
		else printf("That point doesn't exists!\n");
	}
}
